@@ -59,6 +59,7 @@ ac_table = {
             '--static-routing-table',
             '--routing-table',
             '--dynamic-routing',
+            '--extra-search-paths',
             '--zmq-identity',
             '--port-ctrl',
             '--ctrl-with-ipc',
@@ -107,12 +108,14 @@ ac_table = {
             '--expose-public',
             '--shard-id',
             '--pea-id',
+            '--replica-id',
             '--pea-role',
             '--noblock-on-start',
             '--shards',
             '--parallel',
             '--replicas',
             '--install-requirements',
+            '--force-update',
             '--force',
         ],
         'flow': [
@@ -127,6 +130,7 @@ ac_table = {
             '--static-routing-table',
             '--routing-table',
             '--dynamic-routing',
+            '--extra-search-paths',
             '--uses',
             '--env',
             '--inspect',
@@ -146,6 +150,7 @@ ac_table = {
             '--static-routing-table',
             '--routing-table',
             '--dynamic-routing',
+            '--extra-search-paths',
             '--zmq-identity',
             '--port-ctrl',
             '--ctrl-with-ipc',
@@ -200,6 +205,7 @@ ac_table = {
             '--expose-public',
             '--shard-id',
             '--pea-id',
+            '--replica-id',
             '--pea-role',
             '--noblock-on-start',
             '--shards',
@@ -225,12 +231,19 @@ ac_table = {
             '--no-usage',
             '--dockerfile',
             '--tag',
+            '--force-update',
             '--force',
             '--secret',
             '--public',
             '--private',
         ],
-        'hub pull': ['--help', '--no-usage', '--install-requirements', '--force'],
+        'hub pull': [
+            '--help',
+            '--no-usage',
+            '--install-requirements',
+            '--force-update',
+            '--force',
+        ],
         'hub': ['--help', 'new', 'push', 'pull'],
         'help': ['--help'],
         'pea': [
@@ -246,6 +259,7 @@ ac_table = {
             '--static-routing-table',
             '--routing-table',
             '--dynamic-routing',
+            '--extra-search-paths',
             '--zmq-identity',
             '--port-ctrl',
             '--ctrl-with-ipc',
@@ -294,12 +308,14 @@ ac_table = {
             '--expose-public',
             '--shard-id',
             '--pea-id',
+            '--replica-id',
             '--pea-role',
             '--noblock-on-start',
             '--shards',
             '--parallel',
             '--replicas',
             '--install-requirements',
+            '--force-update',
             '--force',
         ],
         'pod': [
@@ -315,6 +331,7 @@ ac_table = {
             '--static-routing-table',
             '--routing-table',
             '--dynamic-routing',
+            '--extra-search-paths',
             '--zmq-identity',
             '--port-ctrl',
             '--ctrl-with-ipc',
@@ -363,12 +380,14 @@ ac_table = {
             '--expose-public',
             '--shard-id',
             '--pea-id',
+            '--replica-id',
             '--pea-role',
             '--noblock-on-start',
             '--shards',
             '--parallel',
             '--replicas',
             '--install-requirements',
+            '--force-update',
             '--force',
             '--uses-before',
             '--uses-after',
