@@ -1,10 +1,12 @@
+from typing import Optional, Dict, Any
+
 from fastapi import APIRouter, Depends
 from fastapi.exceptions import HTTPException
 
-from ..dependencies import FlowDepends
 from ... import Runtime400Exception
-from ...models import DaemonID, ContainerItem, ContainerStoreStatus, FlowModel
+from ..dependencies import FlowDepends
 from ...models.enums import UpdateOperation
+from ...models import DaemonID, ContainerItem, ContainerStoreStatus, FlowModel
 from ...stores import flow_store as store
 
 router = APIRouter(prefix='/flows', tags=['flows'])
@@ -35,7 +37,8 @@ async def _create(flow: FlowDepends = Depends(FlowDepends)):
             workspace_id=flow.workspace_id,
             params=flow.params,
             ports=flow.ports,
-            port_expose=flow.port_expose,
+            envs=flow.envs,
+            device_requests=flow.device_requests,
         )
     except Exception as ex:
         raise Runtime400Exception from ex
@@ -43,17 +46,26 @@ async def _create(flow: FlowDepends = Depends(FlowDepends)):
 
 @router.put(
     path='/{id}',
-    summary='Run an update operation on the Flow object',
+    summary='Trigger an update operation on the Flow object',
     description='Types supported: "rolling_update"',
 )
 async def _update(
     id: DaemonID,
     kind: UpdateOperation,
-    dump_path: str,
     pod_name: str,
-    shards: int = None,
+    dump_path: Optional[str] = None,
+    uses_with: Optional[Dict[str, Any]] = None,
 ):
-    return await store.update(id, kind, dump_path, pod_name, shards)
+    try:
+        if dump_path is not None:
+            if uses_with is not None:
+                uses_with['dump_path'] = dump_path
+            else:
+                uses_with = {'dump_path': dump_path}
+
+        return await store.update(id, kind, pod_name=pod_name, uses_with=uses_with)
+    except Exception as ex:
+        raise Runtime400Exception from ex
 
 
 # order matters! this must be put in front of del {id}
