@@ -1,8 +1,10 @@
-import requests
+from typing import Optional, Dict, Any
+
 from fastapi import Depends, APIRouter, HTTPException
 
-from ..dependencies import PodDepends
 from ... import Runtime400Exception
+from ..dependencies import PodDepends
+from ...models.enums import UpdateOperation
 from ...models import DaemonID, ContainerItem, ContainerStoreStatus, PodModel
 from ...stores import pod_store as store
 
@@ -35,11 +37,35 @@ async def _create(pod: PodDepends = Depends(PodDepends)):
             workspace_id=pod.workspace_id,
             params=pod.params,
             ports=pod.ports,
+            envs=pod.envs,
+            device_requests=pod.device_requests,
         )
     except Exception as ex:
         raise Runtime400Exception from ex
 
 
+@router.put(
+    path='/{id}',
+    summary='Trigger a rolling update on this Pod',
+    description='Types supported: "rolling_update"',
+)
+async def _update(
+    id: DaemonID,
+    kind: UpdateOperation,
+    dump_path: Optional[str] = None,
+    uses_with: Optional[Dict[str, Any]] = None,
+):
+    try:
+        if dump_path is not None:
+            if uses_with is not None:
+                uses_with['dump_path'] = dump_path
+            else:
+                uses_with = {'dump_path': dump_path}
+        return await store.update(id, kind, uses_with=uses_with)
+    except Exception as ex:
+        raise Runtime400Exception from ex
+
+
 @router.delete(
     path='',
     summary='Terminate all running Pods',
@@ -57,7 +83,7 @@ async def _delete(id: DaemonID, workspace: bool = False):
     try:
         await store.delete(id=id, workspace=workspace)
     except KeyError:
-        raise HTTPException(status_code=404, detail=f'{id} not found in {store!r}')
+        raise HTTPException(status_code=404, detail=f'{id} not found in store')
 
 
 @router.get(
@@ -67,16 +93,4 @@ async def _status(id: DaemonID):
     try:
         return store[id]
     except KeyError:
-        raise HTTPException(status_code=404, detail=f'{id} not found in {store!r}')
-
-
-@router.put(path='/{id}/rolling_update', summary='Trigger a rolling update on this Pod')
-async def _rolling_update(id: DaemonID, dump_path: str):
-    try:
-        # TODO: This logic should move to store
-        return requests.put(
-            f'{store[id].metadata.rest_api_uri}/pod/rolling_update',
-            params={'dump_path': dump_path},
-        )
-    except KeyError:
-        raise HTTPException(status_code=404, detail=f'{id} not found in pod store')
+        raise HTTPException(status_code=404, detail=f'{id} not found in store')
