@@ -82,12 +82,13 @@ class ExitFIFO(ExitStack):
         return received_exc and suppressed_exc
 
 
-class BasePod:
+class BasePod(ExitFIFO):
     """A BasePod is an immutable set of peas. They share the same input and output socket.
     Internally, the peas can run with the process/thread backend.
     They can be also run in their own containers on remote machines.
     """
 
+    @abstractmethod
     def start(self) -> 'BasePod':
         """Start to run all :class:`BasePea` in this BasePod.
 
@@ -95,7 +96,17 @@ class BasePod:
             If one of the :class:`BasePea` fails to start, make sure that all of them
             are properly closed.
         """
-        raise NotImplementedError
+        ...
+
+    @abstractmethod
+    async def rolling_update(self, *args, **kwargs):
+        """
+        Roll update the Executors managed by the Pod
+
+            .. # noqa: DAR201
+            .. # noqa: DAR101
+        """
+        ...
 
     @staticmethod
     def _set_upload_files(args):
@@ -255,7 +266,7 @@ class BasePod:
             else:
                 _tail_args.name = f'tail'
             _tail_args.pea_role = PeaRoleType.TAIL
-            _tail_args.num_part = 1 if polling_type.is_push else args.parallel
+            _tail_args.num_part = 1 if polling_type.is_push else args.shards
 
         Pod._set_dynamic_routing_out(_tail_args)
 
@@ -311,13 +322,82 @@ class BasePod:
         ]
 
 
-class Pod(BasePod, ExitFIFO):
-    """A BasePod is an immutable set of peas, which run in parallel. They share the same input and output socket.
+class Pod(BasePod):
+    """A Pod is an immutable set of peas, which run in replicas. They share the same input and output socket.
     Internally, the peas can run with the process/thread backend. They can be also run in their own containers
     :param args: arguments parsed from the CLI
     :param needs: pod names of preceding pods, the output of these pods are going into the input of this pod
     """
 
+    class _ReplicaSet:
+        def __init__(self, pod_args: Namespace, args: List[Namespace]):
+            self.pod_args = pod_args
+            self.args = args
+            self._peas = []
+
+        def activate(self):
+            for pea in self._peas:
+                pea.activate_runtime()
+
+        @property
+        def is_ready(self):
+            return all(p.is_ready.is_set() for p in self._peas)
+
+        def clear_peas(self):
+            self._peas.clear()
+
+        @property
+        def num_peas(self):
+            return len(self._peas)
+
+        def join(self):
+            for pea in self._peas:
+                pea.join()
+
+        def wait_start_success(self):
+            for pea in self._peas:
+                pea.wait_start_success()
+
+        async def rolling_update(
+            self, dump_path: Optional[str] = None, *, uses_with: Optional[Dict] = None
+        ):
+            for i in range(len(self._peas)):
+                old_pea = self._peas[i]
+                old_pea.close()
+                _args = self.args[i]
+                _args.noblock_on_start = True
+                ### BACKWARDS COMPATIBILITY, so THAT DUMP_PATH is in runtime_args
+                _args.dump_path = dump_path
+                ###
+                _args.uses_with = uses_with
+                new_pea = BasePea(_args)
+                new_pea.__enter__()
+                await new_pea.async_wait_start_success()
+                new_pea.activate_runtime()
+                self._peas[i] = new_pea
+
+        def __enter__(self):
+            for _args in self.args:
+                if getattr(self.pod_args, 'noblock_on_start', False):
+                    _args.noblock_on_start = True
+                if (
+                    self.pod_args.replicas == 1
+                ):  # keep backwards compatibility with `workspace` in `Executor`
+                    _args.replica_id = -1
+                self._peas.append(BasePea(_args).start())
+            return self
+
+        def __exit__(self, exc_type, exc_val, exc_tb):
+            closing_exception = None
+            for pea in self._peas:
+                try:
+                    pea.close()
+                except Exception as exc:
+                    if closing_exception is None:
+                        closing_exception = exc
+            if exc_val is None and closing_exception is not None:
+                raise closing_exception
+
     def __init__(
         self,
         args: Union['Namespace', Dict],
@@ -326,15 +406,19 @@ class Pod(BasePod, ExitFIFO):
         super().__init__()
         args.upload_files = BasePod._set_upload_files(args)
         self.args = args
+        # a pod only can have replicas and they can only have polling ANY
+        self.args.polling = PollingType.ANY
         self.needs = (
             needs or set()
         )  #: used in the :class:`jina.flow.Flow` to build the graph
 
+        self.head_pea = None
+        self.tail_pea = None
+        self.replica_set = None
         self.is_head_router = False
         self.is_tail_router = False
         self.deducted_head = None
         self.deducted_tail = None
-        self.peas = []  # type: List['BasePea']
         self.update_pea_args()
         self._activated = False
 
@@ -343,13 +427,21 @@ class Pod(BasePod, ExitFIFO):
         self.join()
 
     def update_pea_args(self):
-        """ Update args of its peas based on Pod args"""
+        """ Update args of all its peas based on Pod args. Including head/tail"""
         if isinstance(self.args, Dict):
             # This is used when a Pod is created in a remote context, where peas & their connections are already given.
             self.peas_args = self.args
         else:
             self.peas_args = self._parse_args(self.args)
 
+    def update_worker_pea_args(self):
+        """ Update args of all its worker peas based on Pod args. Does not touch head and tail"""
+        self.peas_args['peas'] = self._set_peas_args(
+            self.args,
+            head_args=self.peas_args['head'],
+            tail_args=self.peas_args['tail'],
+        )
+
     @property
     def first_pea_args(self) -> Namespace:
         """Return the first non-head/tail pea's args
@@ -467,23 +559,31 @@ class Pod(BasePod, ExitFIFO):
 
         .. # noqa: DAR201
         """
-        return len(self.peas)
+        num_peas = 0
+        if self.head_pea is not None:
+            num_peas += 1
+        if self.tail_pea is not None:
+            num_peas += 1
+        if self.replica_set is not None:  # external pods
+            num_peas += self.replica_set.num_peas
+        return num_peas
 
     def __eq__(self, other: 'BasePod'):
         return self.num_peas == other.num_peas and self.name == other.name
 
-    def _enter_pea(self, pea: 'BasePea') -> None:
-        self.peas.append(pea)
-        self.enter_context(pea)
-
-    def _activate(self):
-        # order is good. Activate from tail to head
-        for pea in reversed(self.peas):
-            pea.activate_runtime()
+    def activate(self):
+        """
+        Activate all peas in this pod
+        """
+        if self.tail_pea is not None:
+            self.tail_pea.activate_runtime()
+        self.replica_set.activate()
+        if self.head_pea is not None:
+            self.head_pea.activate_runtime()
 
         self._activated = True
 
-    def start(self) -> 'BasePod':
+    def start(self) -> 'Pod':
         """
         Start to run all :class:`BasePea` in this BasePod.
 
@@ -493,12 +593,23 @@ class Pod(BasePod, ExitFIFO):
             If one of the :class:`BasePea` fails to start, make sure that all of them
             are properly closed.
         """
-        for _args in self._fifo_args:
+        if self.peas_args['head'] is not None:
+            _args = self.peas_args['head']
             if getattr(self.args, 'noblock_on_start', False):
                 _args.noblock_on_start = True
-            self._enter_pea(BasePea(_args))
+            self.head_pea = BasePea(_args)
+            self.enter_context(self.head_pea)
+        self.replica_set = self._ReplicaSet(self.args, self.peas_args['peas'])
+        self.enter_context(self.replica_set)
+        if self.peas_args['tail'] is not None:
+            _args = self.peas_args['tail']
+            if getattr(self.args, 'noblock_on_start', False):
+                _args.noblock_on_start = True
+            self.tail_pea = BasePea(_args)
+            self.enter_context(self.tail_pea)
+
         if not getattr(self.args, 'noblock_on_start', False):
-            self._activate()
+            self.activate()
         return self
 
     def wait_start_success(self) -> None:
@@ -511,9 +622,12 @@ class Pod(BasePod, ExitFIFO):
                 f'{self.wait_start_success!r} should only be called when `noblock_on_start` is set to True'
             )
         try:
-            for p in self.peas:
-                p.wait_start_success()
-            self._activate()
+            if self.head_pea is not None:
+                self.head_pea.wait_start_success()
+            self.replica_set.wait_start_success()
+            if self.tail_pea is not None:
+                self.tail_pea.wait_start_success()
+            self.activate()
         except:
             self.close()
             raise
@@ -521,13 +635,20 @@ class Pod(BasePod, ExitFIFO):
     def join(self):
         """Wait until all peas exit"""
         try:
-            for p in self.peas:
-                p.join()
-                self._activated = False
+            if self.head_pea is not None:
+                self.head_pea.join()
+            if self.replica_set is not None:
+                self.replica_set.join()
+            if self.tail_pea is not None:
+                self.tail_pea.join()
+            self._activated = False
         except KeyboardInterrupt:
             pass
         finally:
-            self.peas.clear()
+            self.head_pea = None
+            self.tail_pea = None
+            if self.replica_set is not None:
+                self.replica_set.clear_peas()
             self._activated = False
 
     @property
@@ -540,7 +661,35 @@ class Pod(BasePod, ExitFIFO):
 
         .. # noqa: DAR201
         """
-        return all(p.is_ready.is_set() for p in self.peas) and self._activated
+        is_ready = True
+        if self.head_pea is not None:
+            is_ready = self.head_pea.is_ready.is_set()
+        if is_ready:
+            is_ready = self.replica_set.is_ready
+        if is_ready and self.tail_pea is not None:
+            is_ready = self.tail_pea.is_ready.is_set()
+        return is_ready and self._activated
+
+    async def rolling_update(
+        self, dump_path: Optional[str] = None, *, uses_with: Optional[Dict] = None
+    ):
+        """Reload all Peas of this Pod.
+
+        :param dump_path: the dump from which to read the data
+        :param uses_with: a Dictionary of arguments to restart the executor with
+        """
+        # BACKWARDS COMPATIBILITY
+        if dump_path is not None:
+            if uses_with is not None:
+                uses_with['dump_path'] = dump_path
+            else:
+                uses_with = {'dump_path': dump_path}
+        try:
+            await self.replica_set.rolling_update(
+                dump_path=dump_path, uses_with=uses_with
+            )
+        except:
+            raise
 
     @staticmethod
     def _set_peas_args(
@@ -557,18 +706,20 @@ class Pod(BasePod, ExitFIFO):
             ]
         )
 
-        for idx, pea_host in zip(range(args.parallel), cycle(_host_list)):
+        for idx, pea_host in zip(range(args.replicas), cycle(_host_list)):
             _args = copy.deepcopy(args)
-            _args.pea_id = idx
-
-            if args.parallel > 1:
+            # BACKWARDS COMPATIBILITY:
+            # pea_id used to be shard_id so we keep it this way, even though a pea in a BasePod is a replica
+            _args.pea_id = getattr(_args, 'shard_id', 0)
+            _args.replica_id = idx
+            if args.replicas > 1:
                 _args.pea_role = PeaRoleType.PARALLEL
                 _args.identity = random_identity()
 
                 if _args.peas_hosts:
                     _args.host = pea_host
                 if _args.name:
-                    _args.name += f'/pea-{idx}'
+                    _args.name += f'/rep-{idx}'
                 else:
                     _args.name = f'{idx}'
             else:
@@ -580,18 +731,16 @@ class Pod(BasePod, ExitFIFO):
                 _args.port_out = tail_args.port_in
             _args.port_ctrl = helper.random_port()
             _args.socket_out = SocketType.PUSH_CONNECT
-            if args.polling.is_push:
-                if args.scheduling == SchedulerType.ROUND_ROBIN:
-                    _args.socket_in = SocketType.PULL_CONNECT
-                elif args.scheduling == SchedulerType.LOAD_BALANCE:
-                    _args.socket_in = SocketType.DEALER_CONNECT
-                else:
-                    raise ValueError(
-                        f'{args.scheduling} is not supported as a SchedulerType!'
-                    )
 
+            if args.scheduling == SchedulerType.ROUND_ROBIN:
+                _args.socket_in = SocketType.PULL_CONNECT
+            elif args.scheduling == SchedulerType.LOAD_BALANCE:
+                _args.socket_in = SocketType.DEALER_CONNECT
             else:
-                _args.socket_in = SocketType.SUB_CONNECT
+                raise ValueError(
+                    f'{args.scheduling} is not supported as a SchedulerType!'
+                )
+
             if head_args:
                 _args.host_in = get_connect_host(
                     bind_host=head_args.host,
@@ -617,13 +766,13 @@ class Pod(BasePod, ExitFIFO):
 
     def _parse_base_pod_args(self, args):
         parsed_args = {'head': None, 'tail': None, 'peas': []}
-        if getattr(args, 'parallel', 1) > 1:
+        if getattr(args, 'replicas', 1) > 1:
             # reasons to separate head and tail from peas is that they
             # can be deducted based on the previous and next pods
             self.is_head_router = True
             self.is_tail_router = True
-            parsed_args['head'] = BasePod._copy_to_head_args(args, args.polling)
-            parsed_args['tail'] = BasePod._copy_to_tail_args(args, args.polling)
+            parsed_args['head'] = BasePod._copy_to_head_args(args, PollingType.ANY)
+            parsed_args['tail'] = BasePod._copy_to_tail_args(args, PollingType.ANY)
             parsed_args['peas'] = self._set_peas_args(
                 args,
                 head_args=parsed_args['head'],
