@@ -1,7 +1,9 @@
+import csv
+import heapq
 import itertools
 import json
-import heapq
 from abc import abstractmethod
+from collections import Counter
 from collections.abc import MutableSequence, Iterable as Itr
 from contextlib import nullcontext
 from typing import (
@@ -28,10 +30,9 @@ from .search_ops import DocumentArraySearchOpsMixin
 from .traversable import TraversableSequence
 from ..document import Document
 from ..struct import StructView
+from ... import __windows__
 from ...helper import typename
 from ...proto import jina_pb2
-from ... import __windows__
-import csv
 
 try:
     # when protobuf using Cpp backend
@@ -469,25 +470,8 @@ class DocumentArray(
         """
         return len(self) > 0
 
-    def __str__(self):
-
-        content = f'{self.__class__.__name__} has {len(self._pb_body)} items'
-
-        if len(self._pb_body) > 3:
-            content += ' (showing first three)'
-
-        content += ':\n'
-        content += ',\n'.join(str(Document(d)) for d in self._pb_body[:3])
-
-        return content
-
     def __repr__(self):
-        content = ' '.join(
-            f'{k}={v}' for k, v in {'length': len(self._pb_body)}.items()
-        )
-        content += f' at {id(self)}'
-        content = content.strip()
-        return f'<{typename(self)} {content}>'
+        return f'<{typename(self)} (length={len(self._pb_body)}) at {id(self)}>'
 
     def save(
         self, file: Union[str, TextIO, BinaryIO], file_format: str = 'json'
@@ -641,24 +625,22 @@ class DocumentArray(
             return da
 
     @classmethod
-    def load_csv(cls, file: Union[str, BinaryIO]) -> 'DocumentArray':
+    def load_csv(
+        cls,
+        file: Union[str, TextIO],
+        field_resolver: Optional[Dict[str, str]] = None,
+    ) -> 'DocumentArray':
         """Load array elements from a binary file.
 
         :param file: File or filename to which the data is saved.
-
+        :param field_resolver: a map from field names defined in JSON, dict to the field
+            names defined in Document.
         :return: a DocumentArray object
         """
 
-        if hasattr(file, 'read'):
-            file_ctx = nullcontext(file)
-        else:
-            file_ctx = open(file, 'r')
+        from ..document.generators import from_csv
 
-        with file_ctx as fp:
-            da = DocumentArray()
-            for v in csv.DictReader(fp):
-                da.append(Document(v))
-            return da
+        return DocumentArray(from_csv(file, field_resolver=field_resolver))
 
     # Properties for fast access of commonly used attributes
     @property
@@ -746,3 +728,26 @@ class DocumentArray(
     @staticmethod
     def _flatten(sequence) -> 'DocumentArray':
         return DocumentArray(list(itertools.chain.from_iterable(sequence)))
+
+    def get_vocabulary(
+        self, min_freq: int = 1, text_attrs: Tuple[str, ...] = ('text',)
+    ) -> Dict[str, int]:
+        """Get the text vocabulary in a dict that maps from the word to the index from all Documents.
+
+        :param text_attrs: the textual attributes where vocabulary will be derived from
+        :param min_freq: the minimum word frequency to be considered into the vocabulary.
+        :return: a vocabulary in dictionary where key is the word, value is the index. The value is 2-index, where
+            `0` is reserved for padding, `1` is reserved for unknown token.
+        """
+
+        all_tokens = Counter()
+        for d in self:
+            all_tokens.update(d.get_vocabulary(text_attrs=text_attrs))
+
+        # 0 for padding, 1 for unknown
+        return {
+            k: idx
+            for idx, k in enumerate(
+                (k for k, v in all_tokens.items() if v >= min_freq), start=2
+            )
+        }
