@@ -17,28 +17,20 @@ class MatchArray(DocumentArray):
         self._ref_doc = reference_doc
         super().__init__(doc_views)
 
-    def append(self, document: 'Document', copy: bool = True, **kwargs) -> 'Document':
+    def append(self, document: 'Document', **kwargs) -> 'Document':
         """Add a matched document to the current Document.
 
         :param document: Sub-document to be added
-        :param copy: If set, then copy the original Document. Otherwise the original Document may get modified
         :param kwargs: Extra key value arguments
         :return: the newly added sub-document in :class:`Document` view
         :rtype: :class:`Document` view
         """
-        if copy:
-            from ..document import Document
-
-            match = Document(document, copy=True)
-        else:
-            # note: this is faster than Document(document, copy=False)
-            match = document
-
+        super().append(document)
+        match = self[-1]
         match.set_attributes(
             granularity=self.granularity, adjacency=self.adjacency, **kwargs
         )
 
-        super().append(match)
         return match
 
     @property
