@@ -1,17 +1,18 @@
 import base64
 import json
 import mimetypes
+from collections import Counter
 from hashlib import blake2b
 from typing import (
-    Iterable,
-    Union,
-    Dict,
-    Optional,
-    TypeVar,
     Any,
-    Tuple,
+    Dict,
+    Iterable,
     List,
+    Optional,
+    Tuple,
     Type,
+    TypeVar,
+    Union,
     overload,
 )
 
@@ -19,35 +20,27 @@ import numpy as np
 from google.protobuf import json_format
 from google.protobuf.field_mask_pb2 import FieldMask
 
-from .converters import ContentConversionMixin
-from .helper import versioned, VersionedMixin
+from .converters import ContentConversionMixin, _text_to_word_sequence
+from .helper import VersionedMixin, versioned
 from ..mixin import ProtoTypeMixin
-from ..ndarray.generic import NdArray, BaseSparseNdArray
+from ..ndarray.generic import BaseSparseNdArray, NdArray
 from ..score import NamedScore
 from ..score.map import NamedScoreMapping
 from ..struct import StructView
 from ...excepts import BadDocType
-from ...helper import (
-    typename,
-    random_identity,
-    download_mermaid_url,
-    dunder_get,
-)
-from ...importer import ImportExtensions
+from ...helper import download_mermaid_url, dunder_get, random_identity, typename
 from ...logging.predefined import default_logger
 from ...proto import jina_pb2
 
 if False:
+    # fix type-hint complain for sphinx and flake
+    import scipy.sparse
+    import tensorflow
+    import torch
+
     from ..arrays.chunk import ChunkArray
     from ..arrays.match import MatchArray
 
-    from scipy.sparse import coo_matrix
-
-    # fix type-hint complain for sphinx and flake
-    import scipy
-    import tensorflow as tf
-    import torch
-
     ArrayType = TypeVar(
         'ArrayType',
         np.ndarray,
@@ -56,18 +49,11 @@ if False:
         scipy.sparse.bsr_matrix,
         scipy.sparse.csc_matrix,
         torch.sparse_coo_tensor,
-        tf.SparseTensor,
-    )
-
-    SparseArrayType = TypeVar(
-        'SparseArrayType',
-        np.ndarray,
-        scipy.sparse.csr_matrix,
-        scipy.sparse.coo_matrix,
-        scipy.sparse.bsr_matrix,
-        scipy.sparse.csc_matrix,
-        torch.sparse_coo_tensor,
-        tf.SparseTensor,
+        tensorflow.SparseTensor,
+        tensorflow.Tensor,
+        torch.Tensor,
+        jina_pb2.NdArrayProto,
+        NdArray,
     )
 
 __all__ = ['Document', 'DocumentContentType', 'DocumentSourceType']
@@ -146,13 +132,11 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
         self,
         *,
         adjacency: Optional[int] = None,
-        blob: Optional[Union['ArrayType', 'jina_pb2.NdArrayProto', 'NdArray']] = None,
+        blob: Optional['ArrayType'] = None,
         buffer: Optional[bytes] = None,
         chunks: Optional[Iterable['Document']] = None,
         content: Optional[DocumentContentType] = None,
-        embedding: Optional[
-            Union['ArrayType', 'jina_pb2.NdArrayProto', 'NdArray']
-        ] = None,
+        embedding: Optional['ArrayType'] = None,
         granularity: Optional[int] = None,
         id: Optional[str] = None,
         matches: Optional[Iterable['Document']] = None,
@@ -203,9 +187,8 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
                 it builds a view or a copy from it.
         :param copy: when ``document`` is given as a :class:`DocumentProto` object, build a
                 view (i.e. weak reference) from it or a deep copy from it.
-        :param field_resolver: a map from field names defined in ``document`` (JSON, dict) to the field
-                names defined in Protobuf. This is only used when the given ``document`` is
-                a JSON string or a Python dict.
+        :param field_resolver: a map from field names defined in JSON, dict to the field
+            names defined in Document.
         :param kwargs: other parameters to be set _after_ the document is constructed
 
         .. note::
@@ -510,7 +493,7 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
 
     def get_sparse_blob(
         self, sparse_ndarray_cls_type: Type[BaseSparseNdArray], **kwargs
-    ) -> 'SparseArrayType':
+    ) -> 'ArrayType':
         """Return ``blob`` of the content of a Document as an sparse array.
 
         :param sparse_ndarray_cls_type: Sparse class type, such as `SparseNdArray`.
@@ -527,7 +510,7 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
         ).value
 
     @blob.setter
-    def blob(self, value: Union['ArrayType', 'jina_pb2.NdArrayProto', 'NdArray']):
+    def blob(self, value: 'ArrayType'):
         """Set the `blob` to :param:`value`.
 
         :param value: the array value to set the blob
@@ -535,7 +518,7 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
         self._update_ndarray('blob', value)
 
     @property
-    def embedding(self) -> 'SparseArrayType':
+    def embedding(self) -> 'ArrayType':
         """Return ``embedding`` of the content of a Document.
 
          .. note::
@@ -549,7 +532,7 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
 
     def get_sparse_embedding(
         self, sparse_ndarray_cls_type: Type[BaseSparseNdArray], **kwargs
-    ) -> 'SparseArrayType':
+    ) -> 'ArrayType':
         """Return ``embedding`` of the content of a Document as an sparse array.
 
         :param sparse_ndarray_cls_type: Sparse class type, such as `SparseNdArray`.
@@ -566,95 +549,43 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
         ).value
 
     @embedding.setter
-    def embedding(self, value: Union['ArrayType', 'jina_pb2.NdArrayProto', 'NdArray']):
+    def embedding(self, value: 'ArrayType'):
         """Set the ``embedding`` of the content of a Document.
 
         :param value: the array value to set the embedding
         """
         self._update_ndarray('embedding', value)
 
-    def _update_sparse_ndarray(self, k, v, sparse_cls):
-        NdArray(
-            is_sparse=True,
-            sparse_cls=sparse_cls,
-            proto=getattr(self._pb_body, k),
-        ).value = v
+    def _update_ndarray(self, k, v):
+        framework, is_sparse = _get_array_type(v)
 
-    @staticmethod
-    def _check_installed_array_packages():
-        from ... import JINA_GLOBAL
+        if framework == 'jina':
+            _t = NdArray(getattr(self._pb_body, k))
+            _t.is_sparse = v.is_sparse
+            _t.value = v.value
+        elif framework == 'jina_proto':
+            getattr(self._pb_body, k).CopyFrom(v)
 
-        if JINA_GLOBAL.scipy_installed is None:
-            JINA_GLOBAL.scipy_installed = False
-            with ImportExtensions(required=False, pkg_name='scipy'):
-                import scipy
-
-                JINA_GLOBAL.scipy_installed = True
-
-        if JINA_GLOBAL.tensorflow_installed is None:
-            JINA_GLOBAL.tensorflow_installed = False
-            with ImportExtensions(required=False, pkg_name='tensorflow'):
-                import tensorflow
-
-                JINA_GLOBAL.tensorflow_installed = True
-
-        if JINA_GLOBAL.torch_installed is None:
-            JINA_GLOBAL.torch_installed = False
-            with ImportExtensions(required=False, pkg_name='torch'):
-                import torch
-
-                JINA_GLOBAL.torch_installed = True
-
-    def _update_if_sparse(self, k, v):
-
-        from ... import JINA_GLOBAL
-
-        v_valid_sparse_type = False
-        Document._check_installed_array_packages()
-
-        if JINA_GLOBAL.scipy_installed:
-            import scipy
-            import scipy.sparse
-
-            if scipy.sparse.issparse(v):
+        if is_sparse:
+            if framework == 'scipy':
                 from ..ndarray.sparse.scipy import SparseNdArray
-
-                self._update_sparse_ndarray(k=k, v=v, sparse_cls=SparseNdArray)
-                v_valid_sparse_type = True
-
-        if JINA_GLOBAL.tensorflow_installed:
-            import tensorflow
-
-            if isinstance(v, tensorflow.SparseTensor):
+            if framework == 'tensorflow':
                 from ..ndarray.sparse.tensorflow import SparseNdArray
-
-                self._update_sparse_ndarray(k=k, v=v, sparse_cls=SparseNdArray)
-                v_valid_sparse_type = True
-
-        if JINA_GLOBAL.torch_installed:
-            import torch
-
-            if isinstance(v, torch.Tensor) and v.is_sparse:
+            if framework == 'torch':
                 from ..ndarray.sparse.pytorch import SparseNdArray
 
-                self._update_sparse_ndarray(k=k, v=v, sparse_cls=SparseNdArray)
-                v_valid_sparse_type = True
-
-        return v_valid_sparse_type
-
-    def _update_ndarray(self, k, v):
-        if isinstance(v, jina_pb2.NdArrayProto):
-            getattr(self._pb_body, k).CopyFrom(v)
-        elif isinstance(v, np.ndarray):
-            NdArray(getattr(self._pb_body, k)).value = v
-        elif isinstance(v, NdArray):
-            NdArray(getattr(self._pb_body, k)).is_sparse = v.is_sparse
-            NdArray(getattr(self._pb_body, k)).value = v.value
+            NdArray(
+                is_sparse=True,
+                sparse_cls=SparseNdArray,
+                proto=getattr(self._pb_body, k),
+            ).value = v
         else:
-            v_valid_sparse_type = self._update_if_sparse(k, v)
-
-            if not v_valid_sparse_type:
-                raise TypeError(f'{k} is in unsupported type {typename(v)}')
+            if framework == 'numpy':
+                NdArray(getattr(self._pb_body, k)).value = v
+            if framework == 'tensorflow':
+                NdArray(getattr(self._pb_body, k)).value = v.numpy()
+            if framework == 'torch':
+                NdArray(getattr(self._pb_body, k)).value = v.detach().cpu().numpy()
 
     @property
     @versioned
@@ -1062,10 +993,10 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
 
         mermaid_str = (
             """
-                                        %%{init: {'theme': 'base', 'themeVariables': { 'primaryColor': '#FFC666'}}}%%
-                                        classDiagram
-        
-                                                """
+                                            %%{init: {'theme': 'base', 'themeVariables': { 'primaryColor': '#FFC666'}}}%%
+                                            classDiagram
+    
+                                                    """
             + self.__mermaid_str__()
         )
 
@@ -1095,7 +1026,7 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
         showed = False
         if inline_display:
             try:
-                from IPython.display import display, Image
+                from IPython.display import Image, display
 
                 display(Image(url=url))
                 showed = True
@@ -1157,14 +1088,6 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
         else:
             return super().json(*args, **kwargs)
 
-    @property
-    def non_empty_fields(self) -> Tuple[str]:
-        """Return the set fields of the current document that are not empty
-
-        :return: the tuple of non-empty fields
-        """
-        return tuple(field[0].name for field in self.ListFields())
-
     @staticmethod
     def attributes(
         include_proto_fields: bool = True,
@@ -1206,5 +1129,60 @@ class Document(ProtoTypeMixin, VersionedMixin, ContentConversionMixin):
         elif '__' in item:
             value = dunder_get(self._pb_body, item)
         else:
-            raise AttributeError
+            raise AttributeError(f'no attribute named `{item}`')
         return value
+
+    def get_vocabulary(self, text_attrs: Tuple[str, ...] = ('text',)) -> Dict[str, int]:
+        """Get the text vocabulary in a counter dict that maps from the word to its frequency from all :attr:`text_fields`.
+
+        :param text_attrs: the textual attributes where vocabulary will be derived from
+        :return: a vocabulary in dictionary where key is the word, value is the frequency of that word in all text fields.
+        """
+        all_tokens = Counter()
+
+        for f in text_attrs:
+            all_tokens.update(_text_to_word_sequence(getattr(self, f)))
+
+        return all_tokens
+
+
+def _get_array_type(array) -> Tuple[str, bool]:
+    """Get the type of ndarray without importing the framework
+
+    :param array: any array, scipy, numpy, tf, torch, etc.
+    :return: a tuple where the first element represents the framework, the second represents if it is sparse array
+    """
+    module_tags = array.__class__.__module__.split('.')
+    class_name = array.__class__.__name__
+
+    if 'numpy' in module_tags:
+        return 'numpy', False
+
+    if 'jina' in module_tags:
+        if class_name == 'NdArray' or class_name == 'DenseNdArray':
+            return 'jina', False
+        if class_name == 'SparseNdArray':
+            return 'jina', True
+
+    if 'jina_pb2' in module_tags:
+        if class_name == 'DenseNdArrayProto' or class_name == 'NdArrayProto':
+            return 'jina_proto', False
+        if class_name == 'SparseNdArrayProto':
+            return 'jina_proto', True
+
+    if 'tensorflow' in module_tags:
+        if class_name == 'SparseTensor':
+            return 'tensorflow', True
+        if class_name == 'Tensor' or class_name == 'EagerTensor':
+            return 'tensorflow', False
+
+    if 'torch' in module_tags and class_name == 'Tensor':
+        if array.is_sparse:
+            return 'torch', True
+        else:
+            return 'torch', False
+
+    if 'scipy' in module_tags and 'sparse' in module_tags:
+        return 'scipy', True
+
+    raise TypeError(f'can not determine the array type: {module_tags}.{class_name}')
